// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_i18n/wyatt_i18n.dart';

/// {@template i18n}
/// This class is used to store the translations of the application.
/// This entity is used by the I18nDelegate and Flutter's Localizations
/// widget to provide the translations to the application.
/// {@endtemplate}
class I18n extends Entity {
  /// {@macro i18n}
  I18n({
    required this.i18nRepository,
  }) : super();

  final I18nRepository i18nRepository;

  /// Get the translation of the given [key].
  /// If the [key] is not found, the [key] itself is returned.
  /// If the [key] is found, the translation is returned.
  /// If [args] is not null, the translation is formatted with the
  /// given arguments.
  String get(String key, [Map<String, dynamic>? args]) {
    final result = i18nRepository.get(key, args ?? const <String, dynamic>{});

    return result.fold(
      (value) => value,
      (error) => key,
    );
  }

  /// Get the translation of the given [key].
  ///
  /// Note: arguments are not supported.
  String operator [](String key) => get(key);

  /// Get the translation of the given [key].
  String call(String key, [Map<String, dynamic>? args]) => get(key, args);

  /// Load the translations from the given [locale].
  /// If the [locale] is not found, the default locale is loaded.
  Future<void> load(String locale) async {
    await i18nRepository.load(locale: locale);
  }
}
